from datetime import datetime

from PIL.Image import Image as IMG
from PIL.Image import Resampling, Transform
from pil_utils import BuildImage, Text2Image
from pil_utils.gradient import ColorStop, LinearGradient

from meme_generator import add_meme


def fivethousand_choyen(images, texts: list[str], args):
    fontsize = 200
    fontname = "Noto Sans SC"
    text = texts[0]
    pos_x = 40
    pos_y = 220
    imgs: list[tuple[IMG, tuple[int, int]]] = []

    def transform(img: IMG) -> IMG:
        skew = 0.45
        dw = round(img.height * skew)
        return img.transform(
            (img.width + dw, img.height),
            Transform.AFFINE,
            (1, skew, -dw, 0, 1, 0),
            Resampling.BILINEAR,
        )

    def shift(t2m: Text2Image) -> tuple[int, int]:
        return (
            pos_x
            - t2m.lines[0].chars[0].stroke_width
            - max(char.stroke_width for char in t2m.lines[0].chars),
            pos_y - t2m.lines[0].ascent,
        )

    def add_color_text(stroke_width: int, fill: str, pos: tuple[int, int]):
        t2m = Text2Image.from_text(
            text, fontsize, fontname=fontname, stroke_width=stroke_width, fill=fill
        )
        dx, dy = shift(t2m)
        imgs.append((transform(t2m.to_image()), (dx + pos[0], dy + pos[1])))

    def add_gradient_text(
        stroke_width: int,
        dir: tuple[int, int, int, int],
        color_stops: list[tuple[float, tuple[int, int, int]]],
        pos: tuple[int, int],
    ):
        t2m = Text2Image.from_text(
            text, fontsize, fontname=fontname, stroke_width=stroke_width, fill="white"
        )
        mask = transform(t2m.to_image()).convert("L")
        dx, dy = shift(t2m)
        gradient = LinearGradient(
            (dir[0] - dx, dir[1] - dy, dir[2] - dx, dir[3] - dy),
            [ColorStop(*color_stop) for color_stop in color_stops],
        )
        bg = gradient.create_image(mask.size)
        bg.putalpha(mask)
        imgs.append((bg, (dx + pos[0], dy + pos[1])))

    # 黑
    add_color_text(22, "black", (8, 8))
    # 银
    add_gradient_text(
        20,
        (0, 38, 0, 234),
        [
            (0.0, (0, 15, 36)),
            (0.1, (255, 255, 255)),
            (0.18, (55, 58, 59)),
            (0.25, (55, 58, 59)),
            (0.5, (200, 200, 200)),
            (0.75, (55, 58, 59)),
            (0.85, (25, 20, 31)),
            (0.91, (240, 240, 240)),
            (0.95, (166, 175, 194)),
            (1, (50, 50, 50)),
        ],
        (8, 8),
    )
    # 黑
    add_color_text(16, "black", (0, 0))
    # 金
    add_gradient_text(
        10,
        (0, 40, 0, 200),
        [
            (0, (253, 241, 0)),
            (0.25, (245, 253, 187)),
            (0.4, (255, 255, 255)),
            (0.75, (253, 219, 9)),
            (0.9, (127, 53, 0)),
            (1, (243, 196, 11)),
        ],
        (0, 0),
    )
    # 黑
    add_color_text(6, "black", (4, -6))
    # 白
    add_color_text(6, "white", (0, -6))
    # 红
    add_gradient_text(
        4,
        (0, 50, 0, 200),
        [
            (0, (255, 100, 0)),
            (0.5, (123, 0, 0)),
            (0.51, (240, 0, 0)),
            (1, (5, 0, 0)),
        ],
        (0, -6),
    )
    # 红
    add_gradient_text(
        0,
        (0, 50, 0, 200),
        [
            (0, (230, 0, 0)),
            (0.5, (123, 0, 0)),
            (0.51, (240, 0, 0)),
            (1, (5, 0, 0)),
        ],
        (0, -6),
    )

    text = texts[1]
    fontname = "Noto Serif SC"
    pos_x = 300
    pos_y = 480
    # 黑
    add_color_text(22, "black", (10, 4))
    # 银
    add_gradient_text(
        19,
        (0, 320, 0, 506),
        [
            (0, (0, 15, 36)),
            (0.25, (250, 250, 250)),
            (0.5, (150, 150, 150)),
            (0.75, (55, 58, 59)),
            (0.85, (25, 20, 31)),
            (0.91, (240, 240, 240)),
            (0.95, (166, 175, 194)),
            (1, (50, 50, 50)),
        ],
        (10, 4),
    )
    # 黑
    add_color_text(17, "#10193A", (0, 0))
    # 白
    add_color_text(8, "#D0D0D0", (0, 0))
    # 绀
    add_gradient_text(
        7,
        (0, 320, 0, 480),
        [
            (0, (16, 25, 58)),
            (0.03, (255, 255, 255)),
            (0.08, (16, 25, 58)),
            (0.2, (16, 25, 58)),
            (1, (16, 25, 58)),
        ],
        (0, 0),
    )
    # 银
    add_gradient_text(
        0,
        (0, 320, 0, 480),
        [
            (0, (245, 246, 248)),
            (0.15, (255, 255, 255)),
            (0.35, (195, 213, 220)),
            (0.5, (160, 190, 201)),
            (0.51, (160, 190, 201)),
            (0.52, (196, 215, 222)),
            (1.0, (255, 255, 255)),
        ],
        (0, -6),
    )

    img_h = 580
    img_w = max([img.width + pos[0] for img, pos in imgs])
    frame = BuildImage.new("RGBA", (img_w, img_h), "white")
    for img, pos in imgs:
        frame.paste(img, pos, alpha=True)
    return frame.save_jpg()


add_meme(
    "5000choyen",
    fivethousand_choyen,
    min_texts=2,
    max_texts=2,
    default_texts=["我去", "洛天依"],
    keywords=["5000兆"],
    date_created=datetime(2022, 10, 29),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif

img_dir = Path(__file__).parent / "images"


def what_he_wants(images: list[BuildImage], texts: list[str], args):
    date = texts[0] if texts else "今年520"
    text = f"{date}我会给你每个男人都最想要的东西···"
    frame = BuildImage.open(img_dir / "0.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((538, 538), keep_ratio=True, inside=True)
        new_frame = frame.copy()
        try:
            new_frame.draw_text(
                (0, 514, 1024, 614),
                text,
                fill="black",
                max_fontsize=80,
                min_fontsize=20,
                stroke_ratio=0.07,
                stroke_fill="white",
                weight="bold",
                valign="bottom",
            )
        except ValueError:
            raise TextOverLength(date)
        new_frame.paste(img, (486, 616), alpha=True)
        return new_frame

    return make_jpg_or_gif(images, make)


add_meme(
    "what_he_wants",
    what_he_wants,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=["今年520"],
    keywords=["最想要的东西"],
    date_created=datetime(2023, 5, 20),
    date_modified=datetime(2023, 5, 20),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import make_jpg_or_gif


def little_angel(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    img_w, img_h = images[0].convert("RGBA").resize_width(500).size
    frame = BuildImage.new("RGBA", (600, img_h + 230), "white")
    text = "非常可爱！简直就是小天使"
    frame.draw_text(
        (10, img_h + 120, 590, img_h + 185), text, max_fontsize=48, weight="bold"
    )

    ta = "她"
    name = ta
    if texts:
        name = texts[0]
    elif args.user_infos:
        info = args.user_infos[0]
        ta = "他" if info.gender == "male" else "她"
        name = info.name or ta

    text = f"{ta}没失踪也没怎么样  我只是觉得你们都该看一下"
    frame.draw_text(
        (20, img_h + 180, 580, img_h + 215), text, max_fontsize=26, weight="bold"
    )

    text = f"请问你们看到{name}了吗?"
    try:
        frame.draw_text(
            (20, 0, 580, 110), text, max_fontsize=70, min_fontsize=25, weight="bold"
        )
    except ValueError:
        raise TextOverLength(name)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize_width(500)
        return frame.copy().paste(img, (int(300 - img_w / 2), 110), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "little_angel",
    little_angel,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["小天使"],
    date_created=datetime(2022, 1, 1),
    date_modified=datetime(2023, 2, 14),
)



from pathlib import Path
from typing import Optional, Union

import toml
from pydantic import BaseModel

from .compat import model_dump, type_validate_python
from .dirs import get_config_file

config_file_path = get_config_file("config.toml")


class MemeConfig(BaseModel):
    load_builtin_memes: bool = True
    meme_dirs: list[Path] = []
    meme_disabled_list: list[str] = []


class ResourceConfig(BaseModel):
    resource_url: Optional[str] = None
    resource_urls: list[str] = [
        "https://raw.githubusercontent.com/MeetWq/meme-generator/",
        "https://mirror.ghproxy.com/https://raw.githubusercontent.com/MeetWq/meme-generator/",
        "https://cdn.jsdelivr.net/gh/MeetWq/meme-generator@",
        "https://fastly.jsdelivr.net/gh/MeetWq/meme-generator@",
        "https://raw.gitmirror.com/MeetWq/meme-generator/",
    ]


class GifConfig(BaseModel):
    gif_max_size: float = 10
    gif_max_frames: int = 100


class TranslatorConfig(BaseModel):
    baidu_trans_appid: str = ""
    baidu_trans_apikey: str = ""


class ServerConfig(BaseModel):
    host: str = "127.0.0.1"
    port: int = 2233


class LogConfig(BaseModel):
    log_level: Union[int, str] = "INFO"


class Config(BaseModel):
    meme: MemeConfig = MemeConfig()
    resource: ResourceConfig = ResourceConfig()
    gif: GifConfig = GifConfig()
    translate: TranslatorConfig = TranslatorConfig()
    server: ServerConfig = ServerConfig()
    log: LogConfig = LogConfig()

    @classmethod
    def load(cls) -> "Config":
        return type_validate_python(cls, toml.load(config_file_path))

    def dump(self):
        with open(config_file_path, "w", encoding="utf-8") as f:
            toml.dump(model_dump(self), f)


if not config_file_path.exists():
    meme_config = Config()
    config_file_path.write_text("", encoding="utf8")
else:
    meme_config = Config.load()



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

default_text = "你给我的安全感\n远不及{ta}的万分之一"


def safe_sense(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    img = images[0].convert("RGBA").resize((215, 343), keep_ratio=True)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (215, 135))

    ta = "它"
    if args.user_infos:
        gender = args.user_infos[0].gender
        ta = "他" if gender == "male" else "她" if gender == "female" else "它"
    text = texts[0] if texts else default_text.format(ta=ta)
    try:
        frame.draw_text(
            (30, 0, 400, 130),
            text,
            max_fontsize=50,
            allow_wrap=True,
            lines_align="center",
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "safe_sense",
    safe_sense,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["安全感"],
    date_created=datetime(2022, 3, 14),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import translate


def dianzhongdian(images: list[BuildImage], texts: list[str], args):
    if len(texts) == 1:
        text = texts[0]
        trans = translate(text, lang_to="jp")
    else:
        text = texts[0]
        trans = texts[1]

    img = images[0].convert("L").resize_width(500)
    text_img1 = BuildImage.new("RGBA", (500, 60))
    text_img2 = BuildImage.new("RGBA", (500, 35))

    try:
        text_img1.draw_text(
            (20, 0, text_img1.width - 20, text_img1.height),
            text,
            max_fontsize=50,
            min_fontsize=25,
            fill="white",
        )
    except ValueError:
        raise TextOverLength(text)

    try:
        text_img2.draw_text(
            (20, 0, text_img2.width - 20, text_img2.height),
            trans,
            max_fontsize=25,
            min_fontsize=10,
            fill="white",
        )
    except ValueError:
        raise TextOverLength(text)

    frame = BuildImage.new("RGBA", (500, img.height + 100), "black")
    frame.paste(img, alpha=True)
    frame.paste(text_img1, (0, img.height), alpha=True)
    frame.paste(text_img2, (0, img.height + 60), alpha=True)
    return frame.save_jpg()


add_meme(
    "dianzhongdian",
    dianzhongdian,
    min_images=1,
    max_images=1,
    min_texts=1,
    max_texts=2,
    default_texts=["救命啊"],
    keywords=["入典", "典中典", "黑白草图"],
    date_created=datetime(2022, 3, 12),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def mahiro_readbook(images: list[BuildImage], texts, args):
    positions = [
        (0, 118),
        (0, 117),
        (0, 116),
        (0, 116),
        (-3, 116),
        (-7, 117),
    ]
    img = images[0].convert("RGBA").resize((70, 100), keep_ratio=True)
    img = img.perspective(((0, 6), (77, -5), (100, 100), (32, 100)))

    frames = []
    for i in range(48):
        bg = BuildImage.open(img_dir / f"{i}.png")
        frame = BuildImage.new("RGBA", (240, 240), "white")
        frame.paste(img, positions[min(max(i - 16, 0), 5)], alpha=True).paste(
            bg, alpha=True
        )
        frames.append(frame.image)
    return save_gif(frames, 0.08)


add_meme(
    "mahiro_readbook",
    mahiro_readbook,
    min_images=1,
    max_images=1,
    keywords=["真寻看书"],
    tags=MemeTags.mahiro,
    date_created=datetime(2024, 8, 18),
    date_modified=datetime(2024, 8, 18),
)



from datetime import datetime

from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import MemeFeedback, TextOverLength
from meme_generator.utils import make_jpg_or_gif

help_text = "图片“压扁”比例，默认为 2"


class Model(MemeArgsModel):
    ratio: int = Field(2, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-r", "--ratio"],
            args=[ParserArg(name="ratio", value="int")],
            help_text=help_text,
        ),
    ],
)

default_text = "可恶...被人看扁了"


def look_flat(images: list[BuildImage], texts: list[str], args: Model):
    text = texts[0] if texts else default_text
    ratio = args.ratio
    if not 1 <= ratio < images[0].height:
        raise MemeFeedback("请输入合适的“压扁”比例")

    img_w = 500
    text_h = 80
    text_frame = BuildImage.new("RGBA", (img_w, text_h), "white")
    try:
        text_frame.draw_text(
            (10, 0, img_w - 10, text_h),
            text,
            max_fontsize=55,
            min_fontsize=30,
            weight="bold",
        )
    except ValueError:
        raise TextOverLength(text)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize_width(img_w)
        img = img.resize((img_w, img.height // ratio))
        img_h = img.height
        frame = BuildImage.new("RGBA", (img_w, img_h + text_h), "white")
        return frame.paste(img, alpha=True).paste(text_frame, (0, img_h), alpha=True)

    return make_jpg_or_gif(images, make)


add_meme(
    "look_flat",
    look_flat,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    args_type=args_type,
    keywords=["看扁"],
    date_created=datetime(2022, 10, 6),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def paint(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((117, 135), keep_ratio=True)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img.rotate(4, expand=True), (95, 107), below=True)
    return frame.save_jpg()


add_meme(
    "paint",
    paint,
    min_images=1,
    max_images=1,
    keywords=["这像画吗"],
    date_created=datetime(2022, 3, 11),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

default_text = "说完了吗？该走了"


def time_to_go(images: list[BuildImage], texts: list[str], args):
    img = images[0].resize((105, 105), keep_ratio=True)
    frame = BuildImage.open(img_dir / "0.png")
    frame.paste(img, (230, 82), below=True)
    text = texts[0] if texts else default_text
    try:
        frame.draw_text(
            (20, 232, 330, 312),
            text,
            min_fontsize=20,
            max_fontsize=40,
            fill="black",
        )
    except ValueError:
        raise TextOverLength(text)
    return frame.save_jpg()


add_meme(
    "time_to_go",
    time_to_go,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["该走了"],
    date_created=datetime(2024, 9, 4),
    date_modified=datetime(2024, 9, 4),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def roll(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((210, 210))
    # fmt: off
    locs = [
        (87, 77, 0), (96, 85, -45), (92, 79, -90), (92, 78, -135),
        (92, 75, -180), (92, 75, -225), (93, 76, -270), (90, 80, -315)
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(8):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, a = locs[i]
        frame.paste(img.rotate(a), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "roll",
    roll,
    min_images=1,
    max_images=1,
    keywords=["滚"],
    date_created=datetime(2022, 1, 4),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"


def sit_still(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    name = texts[0] if texts else args.user_infos[0].name if args.user_infos else ""
    frame = BuildImage.open(img_dir / "0.png")
    if name:
        try:
            frame.draw_text(
                (100, 170, 600, 330),
                name,
                valign="bottom",
                max_fontsize=75,
                min_fontsize=30,
            )
        except ValueError:
            raise TextOverLength(name)
    img = images[0].convert("RGBA").circle().resize((150, 150)).rotate(-10, expand=True)
    frame.paste(img, (268, 344), alpha=True)
    return frame.save_jpg()


add_meme(
    "sit_still",
    sit_still,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["坐得住", "坐的住"],
    date_created=datetime(2022, 12, 3),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import CommandShortcut, add_meme
from meme_generator.exception import TextOverLength
from meme_generator.tags import MemeTags
from meme_generator.utils import make_png_or_gif

img_dir = Path(__file__).parent / "images"


def oshi_no_ko(images: list[BuildImage], texts: list[str], args):
    name = texts[0] if texts else "网友"

    text_frame1 = BuildImage.open(img_dir / "text1.png")
    text_frame2 = BuildImage.open(img_dir / "text2.png")

    bias_y = 5
    text_frame3 = BuildImage(
        Text2Image.from_text(
            name,
            fontname="HiraginoMin",
            fontsize=150,
            stroke_width=4,
            stroke_fill="white",
        ).to_image()
    ).resize_height(text_frame1.height + bias_y)
    if text_frame3.width > 800:
        raise TextOverLength(name)

    text_frame = BuildImage.new(
        "RGBA",
        (text_frame1.width + text_frame2.width + text_frame3.width, text_frame2.height),
    )
    text_frame.paste(text_frame1, (0, 0), alpha=True).paste(
        text_frame3, (text_frame1.width, bias_y), alpha=True
    ).paste(text_frame2, (text_frame1.width + text_frame3.width, 0), alpha=True)
    text_frame = text_frame.resize_width(663)

    background = BuildImage.open(img_dir / "background.png")
    foreground = BuildImage.open(img_dir / "foreground.png")

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((681, 692), keep_ratio=True)
        return (
            background.copy()
            .paste(img, alpha=True)
            .paste(text_frame, (9, 102 - text_frame.height // 2), alpha=True)
            .paste(foreground, alpha=True)
        )

    return make_png_or_gif(images, make)


add_meme(
    "oshi_no_ko",
    oshi_no_ko,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=["网友"],
    keywords=["我推的网友"],
    shortcuts=[
        CommandShortcut(
            key=r"我推的(?P<name>\S+)",
            args=["{name}"],
            humanized="我推的xx",
        )
    ],
    tags=MemeTags.oshi_no_ko,
    date_created=datetime(2023, 6, 1),
    date_modified=datetime(2023, 6, 23),
)



import random
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.exception import MemeFeedback

img_dir = Path(__file__).parent / "images"


help_text = "图片编号，范围为 1~92"


class Model(MemeArgsModel):
    number: int = Field(0, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    parser_options=[
        ParserOption(
            names=["-n", "--number"],
            args=[ParserArg(name="number", value="int")],
            help_text=help_text,
        ),
    ],
)


def crawl(images: list[BuildImage], texts: list[str], args: Model):
    total_num = 92
    if args.number == 0:
        num = random.randint(1, total_num)
    elif 1 <= args.number <= total_num:
        num = args.number
    else:
        raise MemeFeedback(f"图片编号错误，请选择 1~{total_num}")

    img = images[0].convert("RGBA").circle().resize((100, 100))
    frame = BuildImage.open(img_dir / f"{num:02d}.jpg")
    frame.paste(img, (0, 400), alpha=True)
    return frame.save_jpg()


add_meme(
    "crawl",
    crawl,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["爬"],
    date_created=datetime(2021, 5, 5),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def divorce(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.png")
    img = images[0].convert("RGBA").resize(frame.size, keep_ratio=True)
    frame.paste(img, below=True)
    return frame.save_jpg()


add_meme(
    "divorce",
    divorce,
    min_images=1,
    max_images=1,
    keywords=["离婚协议", "离婚申请"],
    date_created=datetime(2023, 1, 7),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOrNameNotEnough, TextOverLength

img_dir = Path(__file__).parent / "images"


def make_friend(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    img = images[0].convert("RGBA")

    if not texts and not args.user_infos:
        raise TextOrNameNotEnough()
    name = texts[0] if texts else args.user_infos[0].name

    bg = BuildImage.open(img_dir / "0.png")
    frame = img.resize_width(1000)
    frame.paste(
        img.resize_width(250).rotate(9, expand=True),
        (743, frame.height - 155),
        alpha=True,
    )
    frame.paste(
        img.square().resize((55, 55)).rotate(9, expand=True),
        (836, frame.height - 278),
        alpha=True,
    )
    frame.paste(bg, (0, frame.height - 1000), alpha=True)

    text_img = Text2Image.from_text(name, 20, fill="white").to_image()
    if text_img.width > 230:
        raise TextOverLength(name)

    text_img = BuildImage(text_img).rotate(9, expand=True)
    frame.paste(text_img, (710, frame.height - 308), alpha=True)
    return frame.save_jpg()


add_meme(
    "make_friend",
    make_friend,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["交个朋友"],
    date_created=datetime(2022, 3, 9),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def hutao_bite(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square().resize((100, 100))
    frames: list[IMG] = []
    locs = [(98, 101, 108, 234), (96, 100, 108, 237)]
    for i in range(2):
        frame = BuildImage.open(img_dir / f"{i}.png")
        w, h, x, y = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.1)


add_meme(
    "hutao_bite",
    hutao_bite,
    min_images=1,
    max_images=1,
    keywords=["胡桃啃"],
    tags=MemeTags.hutao,
    date_created=datetime(2022, 11, 29),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def thump_wildly(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").resize((122, 122), keep_ratio=True)
    raw_frames = [BuildImage.open(img_dir / f"{i}.png") for i in range(31)]
    for i in range(14):
        raw_frames[i].paste(img, (203, 196), below=True)
    raw_frames[14].paste(img, (207, 239), below=True)
    frames = [frame.image for frame in raw_frames]
    for i in range(6):
        frames.append(frames[0])
    return save_gif(frames, 0.04)


add_meme(
    "thump_wildly",
    thump_wildly,
    min_images=1,
    max_images=1,
    keywords=["捶爆", "爆捶"],
    tags=MemeTags.arknights,
    date_created=datetime(2023, 3, 31),
    date_modified=datetime(2023, 3, 31),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import make_png_or_gif

img_dir = Path(__file__).parent / "images"


def mihoyo(images: list[BuildImage], texts, args):
    mask = BuildImage.new("RGBA", (500, 60), (53, 49, 65, 230))
    logo = BuildImage.open(img_dir / "logo.png").resize_height(50)

    def make(imgs: list[BuildImage]) -> BuildImage:
        img = imgs[0].convert("RGBA").resize((500, 500), keep_ratio=True)
        img.paste(mask, (0, 440), alpha=True)
        img.paste(logo, ((img.width - logo.width) // 2, 445), alpha=True)
        return img.circle_corner(100)

    return make_png_or_gif(images, make)


add_meme(
    "mihoyo",
    mihoyo,
    min_images=1,
    max_images=1,
    keywords=["米哈游"],
    date_created=datetime(2023, 5, 6),
    date_modified=datetime(2023, 5, 6),
)



import re
from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage, Text2Image

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

default_text = "エロ本"


def read_book(images: list[BuildImage], texts: list[str], args):
    frame = BuildImage.open(img_dir / "0.png")
    points = ((0, 108), (1092, 0), (1023, 1134), (29, 1134))
    img = (
        images[0]
        .convert("RGBA")
        .resize((1000, 1100), keep_ratio=True, direction="north")
    )
    cover = img.perspective(points)
    frame.paste(cover, (1138, 1172), below=True)

    text = texts[0] if texts else default_text

    chars = list(" ".join(text.splitlines()))
    pieces: list[BuildImage] = []
    for char in chars:
        piece = BuildImage(
            Text2Image.from_text(char, 150, fill="white", weight="bold").to_image()
        )
        if re.fullmatch(r"[a-zA-Z0-9\s]", char):
            piece = piece.rotate(-90, expand=True)
        else:
            piece = piece.resize_canvas((piece.width, piece.height - 40), "south")
        pieces.append(piece)
    w = max(piece.width for piece in pieces)
    h = sum(piece.height for piece in pieces)
    if w > 265 or h > 3000:
        raise TextOverLength(text)

    text_img = BuildImage.new("RGBA", (w, h))
    h = 0
    for piece in pieces:
        text_img.paste(piece, ((w - piece.width) // 2, h), alpha=True)
        h += piece.height
    if h > 780:
        ratio = 780 / h
        text_img = text_img.resize((int(w * ratio), int(h * ratio)))
    text_img = text_img.rotate(3, expand=True)
    w, h = text_img.size
    frame.paste(text_img, (870 + (240 - w) // 2, 1500 + (780 - h) // 2), alpha=True)

    return frame.save_jpg()


add_meme(
    "read_book",
    read_book,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    default_texts=[default_text],
    keywords=["看书"],
    date_created=datetime(2022, 8, 22),
    date_modified=datetime(2023, 10, 25),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def make_gif(
    key: str,
    texts: list[str],
    pieces: tuple[tuple[int, int], ...],
    fontsize: int = 20,
    padding_x: int = 5,
    padding_y: int = 5,
):
    img = BuildImage.open(img_dir / f"{key}.gif").image
    frames: list[BuildImage] = []
    n_frames = getattr(img, "n_frames", 1)
    for i in range(n_frames):
        img.seek(i)
        frames.append(BuildImage(img.convert("RGB")))

    parts = [frames[start:end] for start, end in pieces]
    for part, text in zip(parts, texts):
        for frame in part:
            try:
                frame.draw_text(
                    (padding_x, 0, frame.width - padding_x, frame.height - padding_y),
                    text,
                    max_fontsize=fontsize,
                    min_fontsize=fontsize,
                    fill="white",
                    stroke_ratio=0.05,
                    stroke_fill="black",
                    valign="bottom",
                )
            except ValueError:
                raise TextOverLength(text)

    return save_gif([frame.image for frame in frames], img.info["duration"] / 1000)


def add_gif_meme(
    key: str,
    keywords: list[str],
    pieces: tuple[tuple[int, int], ...],
    examples: tuple[str, ...],
    date_created: datetime = datetime(2021, 12, 24),
    date_modified: datetime = datetime(2023, 2, 14),
    **kwargs,
):
    def gif_func(images, texts: list[str], args):
        return make_gif(key, texts, pieces, **kwargs)

    text_num = len(pieces)
    add_meme(
        key,
        gif_func,
        min_texts=text_num,
        max_texts=text_num,
        default_texts=list(examples),
        keywords=keywords,
        date_created=date_created,
        date_modified=date_modified,
    )


add_gif_meme(
    "wangjingze",
    ["王境泽"],
    ((0, 9), (12, 24), (25, 35), (37, 48)),
    ("我就是饿死", "死外边 从这里跳下去", "不会吃你们一点东西", "真香"),
)

# fmt: off
add_gif_meme(
    "weisuoyuwei",
    ["为所欲为"],
    ((11, 14), (27, 38), (42, 61), (63, 81), (82, 95), (96, 105), (111, 131), (145, 157), (157, 167),),
    ("好啊", "就算你是一流工程师", "就算你出报告再完美", "我叫你改报告你就要改", "毕竟我是客户", "客户了不起啊", "Sorry 客户真的了不起", "以后叫他天天改报告", "天天改 天天改"),
    fontsize=19,
)
# fmt: on

add_gif_meme(
    "chanshenzi",
    ["馋身子"],
    ((0, 16), (16, 31), (33, 40)),
    ("你那叫喜欢吗？", "你那是馋她身子", "你下贱！"),
    fontsize=18,
)

add_gif_meme(
    "qiegewala",
    ["切格瓦拉"],
    ((0, 15), (16, 31), (31, 38), (38, 48), (49, 68), (68, 86)),
    (
        "没有钱啊 肯定要做的啊",
        "不做的话没有钱用",
        "那你不会去打工啊",
        "有手有脚的",
        "打工是不可能打工的",
        "这辈子不可能打工的",
    ),
)

add_gif_meme(
    "shuifandui",
    ["谁反对"],
    ((3, 14), (21, 26), (31, 38), (40, 45)),
    ("我话说完了", "谁赞成", "谁反对", "我反对"),
    fontsize=19,
)

add_gif_meme(
    "zengxiaoxian",
    ["曾小贤"],
    ((3, 15), (24, 30), (30, 46), (56, 63)),
    ("平时你打电子游戏吗", "偶尔", "星际还是魔兽", "连连看"),
    fontsize=21,
)

add_gif_meme(
    "yalidaye",
    ["压力大爷"],
    ((0, 16), (21, 47), (52, 77)),
    ("外界都说我们压力大", "我觉得吧压力也没有那么大", "主要是28岁了还没媳妇儿"),
    fontsize=21,
)

add_gif_meme(
    "nihaosaoa",
    ["你好骚啊"],
    ((0, 14), (16, 26), (42, 61)),
    ("既然追求刺激", "就贯彻到底了", "你好骚啊"),
    fontsize=17,
)

add_gif_meme(
    "shishilani",
    ["食屎啦你"],
    ((14, 21), (23, 36), (38, 46), (60, 66)),
    ("穿西装打领带", "拿大哥大有什么用", "跟着这样的大哥", "食屎啦你"),
    fontsize=17,
)

add_gif_meme(
    "wunian",
    ["五年怎么过的"],
    ((11, 20), (35, 50), (59, 77), (82, 95)),
    (
        "五年",
        "你知道我这五年是怎么过的吗",
        "我每天躲在家里玩贪玩蓝月",
        "你知道有多好玩吗",
    ),
    fontsize=16,
)

add_gif_meme(
    "maikease",
    ["麦克阿瑟说"],
    ((0, 22), (24, 46), (48, 70), (72, 84)),
    (
        "美国前五星上将麦克阿瑟",
        "曾这样评价道",
        "如果让我去阻止xxx",
        "那么我宁愿去阻止上帝",
    ),
    date_created=datetime(2023, 7, 30),
    date_modified=datetime(2023, 7, 30),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def tom_tease(images: list[BuildImage], texts, args):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize((400, 350), keep_ratio=True)
            img = img.perspective(((0, 100), (290, 0), (290, 370), (0, 335)))
            bg = BuildImage.open(img_dir / f"{i}.png")
            frame = BuildImage.new("RGBA", bg.size, "white")
            frame.paste(bg).paste(img, (258, -12), below=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 11, 0.2, FrameAlignPolicy.extend_first
    )


add_meme(
    "tom_tease",
    tom_tease,
    min_images=1,
    max_images=1,
    keywords=["汤姆嘲笑"],
    tags=MemeTags.tom,
    date_created=datetime(2024, 1, 19),
    date_modified=datetime(2024, 1, 19),
)



from datetime import datetime
from typing import Literal

from arclet.alconna import store_value
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

help_text = "循环方向，包含 left、right、top、bottom"


class Model(MemeArgsModel):
    direction: Literal["left", "right", "top", "bottom"] = Field(
        "top", description=help_text
    )


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[
        Model(direction="left"),
        Model(direction="right"),
        Model(direction="top"),
        Model(direction="bottom"),
    ],
    parser_options=[
        ParserOption(
            names=["-d", "--direction"],
            args=[ParserArg(name="direction", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--left", "左"],
            dest="direction",
            action=store_value("left"),
        ),
        ParserOption(
            names=["--right", "右"],
            dest="direction",
            action=store_value("right"),
        ),
        ParserOption(
            names=["--top", "上"],
            dest="direction",
            action=store_value("top"),
        ),
        ParserOption(
            names=["--bottom", "下"],
            dest="direction",
            action=store_value("bottom"),
        ),
    ],
)


def loop(images: list[BuildImage], texts, args: Model):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA")
            width, height = img.size

            if args.direction in ["top", "bottom"]:
                extend_img = BuildImage.new("RGBA", (width, height * 2), "white")
                extend_img.paste(img, (0, 0))
                extend_img.paste(img, (0, height))

                dh = round(height / 30 * i)
                if args.direction == "top":
                    return extend_img.crop((0, dh, width, dh + height))
                else:
                    return extend_img.crop((0, height - dh, width, height * 2 - dh))

            else:
                extend_img = BuildImage.new("RGBA", (width * 2, height), "white")
                extend_img.paste(img, (0, 0))
                extend_img.paste(img, (width, 0))

                dw = round(width / 30 * i)
                if args.direction == "left":
                    return extend_img.crop((dw, 0, dw + width, height))
                else:
                    return extend_img.crop((width - dw, 0, width * 2 - dw, height))

        return make

    return make_gif_or_combined_gif(
        images, maker, 30, 0.05, FrameAlignPolicy.extend_loop
    )


add_meme(
    "loop",
    loop,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["循环"],
    date_created=datetime(2024, 7, 14),
    date_modified=datetime(2024, 8, 15),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"


def telescope(images: list[BuildImage], texts, args):
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").square()
            img_big = img.resize((600, 600))
            img_small = img.resize((230, 230))
            frame = BuildImage.open(img_dir / f"{i}.png")
            if 4 <= i < 18:
                x = -167 + (i - 4) * 4
                y = -361 + (i - 4) * 7
                frame.paste(img_big, (x, y), below=True)
            elif 23 <= i < 38:
                x = -90 + (i - 23) * 5
                y = -245 + (i - 23) * 5
                frame.paste(img_big, (x, y), below=True)
            elif 43 <= i < 46:
                x = -15
                y = -210
                frame.paste(img_big, (x, y), below=True)
            elif 46 <= i < 57:
                x = 8
                y = -21
                frame.paste(img_small, (x, y), below=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 69, 0.1, FrameAlignPolicy.extend_first
    )


add_meme(
    "telescope",
    telescope,
    min_images=1,
    max_images=1,
    keywords=["望远镜"],
    date_created=datetime(2024, 1, 18),
    date_modified=datetime(2024, 1, 18),
)



from datetime import datetime

from PIL import ImageFilter
from pil_utils import BuildImage, Text2Image
from pil_utils.gradient import ColorStop, LinearGradient

from meme_generator import MemeArgsModel, add_meme
from meme_generator.exception import TextOrNameNotEnough, TextOverLength


def ask(images: list[BuildImage], texts: list[str], args: MemeArgsModel):
    if not texts and not args.user_infos:
        raise TextOrNameNotEnough()

    name = texts[0] if texts else args.user_infos[0].name
    ta = "他" if args.user_infos and args.user_infos[0].gender == "male" else "她"

    img = images[0].resize_width(640)
    img_w, img_h = img.size
    gradient_h = 150
    gradient = LinearGradient(
        (0, 0, 0, gradient_h),
        [ColorStop(0, (0, 0, 0, 220)), ColorStop(1, (0, 0, 0, 30))],
    )
    gradient_img = gradient.create_image((img_w, gradient_h))
    mask = BuildImage.new("RGBA", img.size)
    mask.paste(gradient_img, (0, img_h - gradient_h), alpha=True)
    mask = mask.filter(ImageFilter.GaussianBlur(radius=3))
    img.paste(mask, alpha=True)

    start_w = 20
    start_h = img_h - gradient_h + 5
    text1 = name
    text2 = f"{name}不知道哦。"
    text2img1 = Text2Image.from_text(text1, 28, weight="bold")
    text2img2 = Text2Image.from_text(text2, 28, weight="bold")
    img.draw_text(
        (start_w + 40 + (text2img2.width - text2img1.width) // 2, start_h),
        text1,
        fontsize=28,
        fill="orange",
        weight="bold",
    )
    img.draw_text(
        (start_w + 40, start_h + text2img1.height + 10),
        text2,
        fontsize=28,
        fill="white",
        weight="bold",
    )

    line_h = start_h + text2img1.height + 5
    img.draw_line(
        (start_w, line_h, start_w + text2img2.width + 80, line_h),
        fill="orange",
        width=2,
    )

    sep_w = 30
    sep_h = 80
    frame = BuildImage.new("RGBA", (img_w + sep_w * 2, img_h + sep_h * 2), "white")
    try:
        frame.draw_text(
            (sep_w, 0, img_w + sep_w, sep_h),
            f"让{name}告诉你吧",
            max_fontsize=35,
            halign="left",
        )
        frame.draw_text(
            (sep_w, img_h + sep_h, img_w + sep_w, img_h + sep_h * 2),
            f"啊这，{ta}说不知道",
            max_fontsize=35,
            halign="left",
        )
    except ValueError:
        raise TextOverLength(name)
    frame.paste(img, (sep_w, sep_h), alpha=True)
    return frame.save_png()


add_meme(
    "ask",
    ask,
    min_images=1,
    max_images=1,
    min_texts=0,
    max_texts=1,
    keywords=["问问"],
    date_created=datetime(2022, 2, 23),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from typing import Literal, NamedTuple

from arclet.alconna import store_value
from PIL.Image import Image as IMG
from PIL.Image import Transpose
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import (
    MemeArgsModel,
    MemeArgsType,
    ParserArg,
    ParserOption,
    add_meme,
)
from meme_generator.utils import save_gif

help_text = "鬼畜对称方向，包含 left、right、top、bottom"


class Model(MemeArgsModel):
    direction: Literal["left", "right", "top", "bottom"] = Field(
        "left", description=help_text
    )


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[
        Model(direction="left"),
        Model(direction="right"),
        Model(direction="top"),
        Model(direction="bottom"),
    ],
    parser_options=[
        ParserOption(
            names=["-d", "--direction"],
            args=[ParserArg(name="direction", value="str")],
            help_text=help_text,
        ),
        ParserOption(
            names=["--left", "左"], dest="direction", action=store_value("left")
        ),
        ParserOption(
            names=["--right", "右"], dest="direction", action=store_value("right")
        ),
        ParserOption(
            names=["--top", "上"], dest="direction", action=store_value("top")
        ),
        ParserOption(
            names=["--bottom", "下"], dest="direction", action=store_value("bottom")
        ),
    ],
)


def guichu(images: list[BuildImage], texts, args: Model):
    img = images[0].convert("RGBA")
    img_w, img_h = img.size

    class Mode(NamedTuple):
        method: Transpose
        size1: tuple[int, int, int, int]
        pos1: tuple[int, int]
        size2: tuple[int, int, int, int]
        pos2: tuple[int, int]

    modes: dict[str, Mode] = {
        "left": Mode(
            Transpose.FLIP_LEFT_RIGHT,
            (0, 0, img_w // 2, img_h),
            (0, 0),
            (img_w // 2, 0, img_w // 2 * 2, img_h),
            (img_w // 2, 0),
        ),
        "right": Mode(
            Transpose.FLIP_LEFT_RIGHT,
            (img_w // 2, 0, img_w // 2 * 2, img_h),
            (img_w // 2, 0),
            (0, 0, img_w // 2, img_h),
            (0, 0),
        ),
        "top": Mode(
            Transpose.FLIP_TOP_BOTTOM,
            (0, 0, img_w, img_h // 2),
            (0, 0),
            (0, img_h // 2, img_w, img_h // 2 * 2),
            (0, img_h // 2),
        ),
        "bottom": Mode(
            Transpose.FLIP_TOP_BOTTOM,
            (0, img_h // 2, img_w, img_h // 2 * 2),
            (0, img_h // 2),
            (0, 0, img_w, img_h // 2),
            (0, 0),
        ),
    }
    mode = modes[args.direction]

    img_flip = img.transpose(mode.method)
    img_symmetric = BuildImage.new("RGBA", img.size)
    img_symmetric.paste(img.crop(mode.size1), mode.pos1, alpha=True)
    img_symmetric.paste(img_flip.crop(mode.size2), mode.pos2, alpha=True)
    img_symmetric_big = BuildImage.new("RGBA", img.size)
    img_symmetric_big.paste(
        img_symmetric.copy().resize_width(img_w * 2), (-img_w // 2, -img_h // 2)
    )

    frames: list[IMG] = []
    frames += (
        ([img.image] * 3 + [img_flip.image] * 3) * 3
        + [img.image, img_flip.image] * 3
        + ([img_symmetric.image] * 2 + [img_symmetric_big.image] * 2) * 2
    )

    return save_gif(frames, 0.20)


add_meme(
    "guichu",
    guichu,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["鬼畜"],
    date_created=datetime(2023, 7, 19),
    date_modified=datetime(2023, 7, 19),
)



from datetime import datetime
from pathlib import Path

from arclet.alconna import store_true
from pil_utils import BuildImage
from pydantic import Field

from meme_generator import MemeArgsModel, MemeArgsType, ParserOption, add_meme
from meme_generator.tags import MemeTags
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif

img_dir = Path(__file__).parent / "images"

help_text = "是否将图片变为圆形"


class Model(MemeArgsModel):
    circle: bool = Field(False, description=help_text)


args_type = MemeArgsType(
    args_model=Model,
    args_examples=[Model(circle=False), Model(circle=True)],
    parser_options=[
        ParserOption(
            names=["--circle", "圆"],
            default=False,
            action=store_true,
            help_text=help_text,
        ),
    ],
)


def kirby_hammer(images: list[BuildImage], texts, args: Model):
    # fmt: off
    positions = [
        (318, 163), (319, 173), (320, 183), (317, 193), (312, 199),
        (297, 212), (289, 218), (280, 224), (278, 223), (278, 220),
        (280, 215), (280, 213), (280, 210), (280, 206), (280, 201),
        (280, 192), (280, 188), (280, 184), (280, 179)
    ]
    # fmt: on
    def maker(i: int) -> Maker:
        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA")
            if args.circle:
                img = img.circle()
            img = img.resize_height(80)
            if img.width < 80:
                img = img.resize((80, 80), keep_ratio=True)
            frame = BuildImage.open(img_dir / f"{i}.png")
            if i <= 18:
                x, y = positions[i]
                x = x + 40 - img.width // 2
                frame.paste(img, (x, y), alpha=True)
            elif i <= 39:
                x, y = positions[18]
                x = x + 40 - img.width // 2
                frame.paste(img, (x, y), alpha=True)
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, 62, 0.05, FrameAlignPolicy.extend_loop
    )


add_meme(
    "kirby_hammer",
    kirby_hammer,
    min_images=1,
    max_images=1,
    args_type=args_type,
    keywords=["卡比锤", "卡比重锤"],
    tags=MemeTags.kirby,
    date_created=datetime(2022, 11, 8),
    date_modified=datetime(2023, 2, 14),
)



import math
from datetime import datetime

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import FrameAlignPolicy, Maker, make_gif_or_combined_gif


def wave(images: list[BuildImage], texts, args):
    img_w = min(max(images[0].width, 360), 720)
    period = img_w / 6
    amp = img_w / 60
    frame_num = 8

    def maker(i: int) -> Maker:
        def sin(x):
            return (
                amp * math.sin(2 * math.pi / period * (x + i * period / frame_num)) / 2
            )

        def make(imgs: list[BuildImage]) -> BuildImage:
            img = imgs[0].convert("RGBA").resize_width(img_w)
            img_h = img.height
            frame = img.copy()
            for i in range(img_w):
                for j in range(img_h):
                    dx = int(sin(i) * (img_h - j) / img_h)
                    dy = int(sin(j) * j / img_h)
                    if 0 <= i + dx < img_w and 0 <= j + dy < img_h:
                        frame.image.putpixel(
                            (i, j),
                            img.image.getpixel((i + dx, j + dy)),  # type: ignore
                        )

            frame = frame.resize_canvas((int(img_w - amp), int(img_h - amp)))
            return frame

        return make

    return make_gif_or_combined_gif(
        images, maker, frame_num, 0.01, FrameAlignPolicy.extend_loop
    )


add_meme(
    "wave",
    wave,
    min_images=1,
    max_images=1,
    keywords=["波纹"],
    date_created=datetime(2022, 10, 26),
    date_modified=datetime(2023, 2, 14),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme

img_dir = Path(__file__).parent / "images"


def tomb_yeah(images: list[BuildImage], texts, args):
    frame = BuildImage.open(img_dir / "0.jpg").convert("RGBA")
    frame.paste(
        images[0].convert("RGBA").circle().resize((145, 145)), (138, 265), alpha=True
    )
    if len(images) > 1:
        frame.paste(
            images[1].convert("RGBA").circle().rotate(30).resize((145, 145)),
            (371, 312),
            alpha=True,
        )
    return frame.save_jpg()


add_meme(
    "tomb_yeah",
    tomb_yeah,
    min_images=1,
    max_images=2,
    keywords=["上坟", "坟前比耶"],
    date_created=datetime(2023, 11, 12),
    date_modified=datetime(2023, 11, 12),
)



from datetime import datetime
from pathlib import Path

from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.exception import TextOverLength

img_dir = Path(__file__).parent / "images"

default_text = "这是十二生肖中的鸡"


def this_chicken(images: list[BuildImage], texts, args):
    text = texts[0] if texts else default_text
    img = images[0].convert("RGBA").resize((640, 640), keep_ratio=True)

    frame = BuildImage.open(img_dir / "0.png")
    try:
        frame.draw_text(
            (0, 900, 1440, 1080),
            text,
            max_fontsize=80,
            min_fontsize=40,
            fill="white",
            stroke_ratio=1 / 15,
            stroke_fill="black",
        )
    except ValueError:
        raise TextOverLength(text)
    frame.paste(
        img.perspective(((507, 0), (940, 351), (383, 625), (0, 256))),
        (201, 201),
        below=True,
    )
    return frame.save_jpg()


add_meme(
    "this_chicken",
    this_chicken,
    min_images=1,
    max_images=1,
    max_texts=1,
    default_texts=[default_text],
    keywords=["这是鸡", "🐔"],
    date_created=datetime(2023, 11, 12),
    date_modified=datetime(2024, 1, 18),
)



from datetime import datetime
from pathlib import Path

from PIL.Image import Image as IMG
from pil_utils import BuildImage

from meme_generator import add_meme
from meme_generator.utils import save_gif

img_dir = Path(__file__).parent / "images"


def hammer(images: list[BuildImage], texts, args):
    img = images[0].convert("RGBA").square()
    # fmt: off
    locs = [
        (62, 143, 158, 113), (52, 177, 173, 105), (42, 192, 192, 92), (46, 182, 184, 100),
        (54, 169, 174, 110), (69, 128, 144, 135), (65, 130, 152, 124),
    ]
    # fmt: on
    frames: list[IMG] = []
    for i in range(7):
        frame = BuildImage.open(img_dir / f"{i}.png")
        x, y, w, h = locs[i]
        frame.paste(img.resize((w, h)), (x, y), below=True)
        frames.append(frame.image)
    return save_gif(frames, 0.07)


add_meme(
    "hammer",
    hammer,
    min_images=1,
    max_images=1,
    keywords=["锤"],
    date_created=datetime(2022, 4, 20),
    date_modified=datetime(2023, 2, 14),
)
